@@ -29,8 +29,13 @@ from time import gmtime, strftime
 import os, sys
 from git import Repo
 
+
+class FabricException(Exception):
+    pass
+
+
 env.user="ubuntu"
-TRUTH_VALUES = [True, 1, '1', 'true', 't', 'yes', 'y']
+TRUTH_VALUES = ['True', 'TRUE', '1', 'true', 't', 'Yes', 'YES', 'yes', 'y'] # arguments in fab are always strings
 
 import boto, urllib2
 from   boto.ec2 import connect_to_region
@@ -49,16 +54,16 @@ AWSAddress = collections.namedtuple('AWSAddress', 'name publicdns privateip shar
 #  action is the deferred action needed, such as "deploy", "security-updates", etc.
 #  region is aws region
 @task
-def set_hosts(type,primary=None,appname=None,action=None,region=None):
+def set_hosts(type,primary=None,appname=None,action=None,region=None,shard=all):
     if appname is None:
         local('echo "ERROR: appname option is not set"')
     if region is None:
         local('echo "ERROR: region option is not set"')
     environment = os.environ["AWS_ENVIRONMENT"]
-    awsaddresses    = _get_awsaddress(type, primary, environment, appname, action, region)
+    awsaddresses    = _get_awsaddress(type, primary, environment, appname, action, region, shard)
 
     env.hosts = list( item.publicdns for item in awsaddresses)
-    env.host_names = list (item.name for item in awsaddresses)
+    env.host_names = list(item.name for item in awsaddresses)
 
     _log_hosts(awsaddresses)
 
@@ -66,13 +71,13 @@ def set_hosts(type,primary=None,appname=None,action=None,region=None):
 # set_one_host picks a single instance out of the set.
 #  filters are the same as with set_hosts.
 @task
-def set_one_host(type,primary=None,appname=None,action=None,region=None):
+def set_one_host(type,primary=None,appname=None,action=None,region=None,shard=all):
     if appname is None:
         local('echo "ERROR: appname option is not set"')
     if region is None:
         local('echo "ERROR: region option is not set"')
     environment = os.environ["AWS_ENVIRONMENT"]
-    awsaddresses    = _get_awsaddress(type, primary, environment, appname, action, region)
+    awsaddresses    = _get_awsaddress(type, primary, environment, appname, action, region, shard)
 
     awsaddresses = [awsaddresses[0]]
 
@@ -82,12 +87,34 @@ def set_one_host(type,primary=None,appname=None,action=None,region=None):
     _log_hosts(awsaddresses)
 
 
+@task
+def set_one_host_per_shard(type,primary=None,appname=None,action=None,region=None,shard=all):
+    if appname is None:
+        local('echo "ERROR: appname option is not set"')
+    if region is None:
+        local('echo "ERROR: region option is not set"')
+    environment = os.environ["AWS_ENVIRONMENT"]
+    awsaddresses    = _get_awsaddress(type, primary, environment, appname, action, region, shard)
+
+    pruned_list = []
+    for ahost in awsaddresses:
+        if not next((True for bhost in pruned_list if ahost.shard == bhost.shard), False):
+            pruned_list.append(ahost)
+
+    env.hosts = list( item.publicdns for item in pruned_list)
+    env.host_names = list(item.name for item in pruned_list)
+
+    _log_hosts(pruned_list)
+
+
+
 def _log_hosts(awsaddresses):
     logger.info("")
     logger.info("Instances to operate upon - name, public dns, private ip, shard")
     logger.info("---------------------------------------------------------------")
     for instance in awsaddresses:
-        logger.info("%s  %s  %s  %s\n", instance.name, instance.publicdns, instance.privateip, instance.shard)
+        logger.info("%s  %s  %s  %s", instance.name, instance.publicdns, instance.privateip, instance.shard)
+    logger.info("")
 
 
 @task
@@ -119,7 +146,7 @@ def show_environment():
     run('env')
 
 # Private method to get public DNS name for instance with given tag key and value pair
-def _get_awsaddress(type,primary, environment,appname,action,region):
+def _get_awsaddress(type,primary, environment,appname,action,region, shard):
     awsaddresses = []
     connection   = _create_connection(region)
     aws_tags = {"tag:Type" : type, "tag:Env" : environment, "tag:App" : appname, "instance-state-name" : "running"}
@@ -129,12 +156,14 @@ def _get_awsaddress(type,primary, environment,appname,action,region):
         aws_tags["tag:Primary"] = primary
     logger.info("Filtering via tags=%s", aws_tags)
     instances = connection.get_only_instances(filters = aws_tags)
+    shards = [ e for e in shard.split(',') ]
     for instance in instances:
         if instance.public_dns_name:                                # make sure there's really an instance here
             shardt = ("None" if not 'Shard' in instance.tags else instance.tags['Shard'])                    
             awsaddress = AWSAddress( name=instance.tags['Name'], publicdns=instance.public_dns_name, 
                 privateip=instance.private_ip_address, shard=shardt)
-            awsaddresses.append(awsaddress)
+            if (shard == 'all') or (shardt in shards):
+                awsaddresses.append(awsaddress)
     return awsaddresses
 
 # Private method for getting AWS connection
@@ -193,6 +222,7 @@ def pip_install():
 
 @task
 def download_nltk_data():
+    run("echo 'loading NLTK data specified in nltk.txt'")
     sudo_app('if [[ -f nltk.txt ]]; then mkdir -p /usr/share/nltk_data/; cat nltk.txt | while read -r line; do python -m nltk.downloader -d /usr/share/nltk_data ${line}; done; fi')
 
 @task
@@ -228,10 +258,14 @@ def codeversioner():
         sudo(cmd)
 
 @task
-def deploycode(branch,doCollectStatic=True):
+def deploycode(branch,nltkLoad="False",doCollectStatic="True"):
     tar_from_git(branch)
     remote_inflate_code()
     pip_install()
+ 
+    if nltkLoad in TRUTH_VALUES:
+        download_nltk_data()
+
     if doCollectStatic in TRUTH_VALUES:
         collect_static()
     else:
@@ -239,11 +273,14 @@ def deploycode(branch,doCollectStatic=True):
 
 @task
 @parallel
-def deployParallel(doCollectStatic=True):
+def deployParallel(nltkLoad="False",doCollectStatic="True"):
     remote_inflate_code()
     pip_install()
+    if nltkLoad in TRUTH_VALUES:
+        download_nltk_data()
+
     if doCollectStatic in TRUTH_VALUES:
-        collectstatic()
+        collect_static()
     else:
         sudo('cp -r ' + '/usr/local/opt/python/lib/python2.7/site-packages' + '/django/contrib/admin/static/admin /data/deploy/pending/static/')
 
@@ -252,6 +289,7 @@ def dbmigrate_docker(containerid,codepath='/data/deploy/current'):
     run('docker exec -it %s /bin/bash -c "cd /data/deploy/current && python manage.py migrate --noinput --ignore-ghost-migrations"' % containerid)
 
 @task
+@parallel
 def dbmigrate(migrateOptions=None):
     cmdToRun = "cd /data/deploy/pending && python manage.py migrate --noinput"
 
@@ -282,6 +320,16 @@ def restart_web():
 
 @task
 @parallel
+def reload_worker(async="djangorq"):
+    swap_code()
+    if async == "djangorq":
+        reload_djangorq()
+    elif async == "celery":
+        restart_celery
+    else:
+        logger.info("Specified async facility not supported")
+@task
+@parallel
 def restart_worker(async="djangorq"):
     swap_code()
     if async == "djangorq":
@@ -308,6 +356,10 @@ def reload_nginx():
 def reload_uwsgi():
     sudo("/bin/bash -c 'echo c > /tmp/uwsgififo'")
 
+@task 
+def reload_djangorq():
+    sudo("for process in $(ps -ef|grep rq|grep -v grep|awk '{print $2}'); do kill -INT ${process}; done")
+
 @task
 def restart_nginx():
     sudo("%s restart nginx" % supervisor)
@@ -338,12 +390,24 @@ def sudo_cmd(cmdToRun):
     sudo(cmdToRun)
 
 @task
-def run_app(cmdToRun):
-    run('cd /data/deploy/pending ; ' + cmdToRun)
+def run_app(cmdToRun, stopOnError="True"):
+    if stopOnError in TRUTH_VALUES:
+        with cd('/data/deploy/pending'):
+            run(cmdToRun)
+    else:
+        with settings(abort_exception = FabricException):
+            try:
+                with cd('/data/deploy/pending'):
+                    run(cmdToRun)
+            except FabricException:
+                pass
 
 @task
 def sudo_app(cmdToRun):
-    sudo('cd /data/deploy/pending ; ' + cmdToRun)
+    with cd('/data/deploy/pending'):
+        sudo(cmdToRun)
+
+
 
 # Obtain git_sha from Jenkins git plugin, make sure it's passed along with the code so that
 # uwsgi, djangorq, and and celery can make use of it (e.g. to pass to Sentry for releases)
